@@ -2,7 +2,7 @@
 *          sdop - Simple DocBook Processor       *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 2009, 2011 */
+/* Copyright (c) Philip Hazel, 2009, 2011, 2012 */
 
 /* Error handling routines */
 
@@ -175,10 +175,11 @@ static error_struct error_data[] = {
 { ec_disaster, "internal error: absolute page number unset for reference to "
                "%s while generating output" },
 { ec_disaster, "internal error: absolute page y position unset for reference "
-               "to %s while generating output" }
+               "to %s while generating output" },
+{ ec_disaster, "too many nested lists" }
 };
 
-#define error_maxerror 113
+#define error_maxerror 114
 
 
 
