@@ -2,7 +2,7 @@
 *          sdop - Simple DocBook Processor       *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 2009 */
+/* Copyright (c) Philip Hazel, 2009, 2011 */
 
 /* This module contains functions related to fonts. */
 
@@ -11,6 +11,44 @@
 
 
 
+/*************************************************
+*             Find a bullet value                *
+*************************************************/
+
+/* This is used for identified lists, both in this module and also from write.c
+when actually outputting.
+
+Argument:   the <identifiedlist> item
+Returns:    zero if there is no "mark" parameter
+            a positive value for the bullet character
+*/
+
+int
+font_get_bullet(item *i)
+{
+uschar *markname;
+paramstr *p = misc_param_find(i, US"mark");
+
+if (p == NULL) return 0;
+markname = p->value;
+
+if (Ustrcmp(markname, "bullet") == 0) return BULLET_BULLET;
+else if (Ustrcmp(markname, "dash") == 0) return BULLET_DASH;
+else if (Ustrcmp(markname, "opencircle") == 0) return BULLET_OCIRCLE;
+else if (markname[0] == 'U' && markname[1] == '+')
+  {
+    /* Making this uschar instead of char causes gcc to mutter
+    "dereferencing type-punned pointer will break strict-aliasing rules"
+    and there seems to other way to stop it. */
+  char *endptr;
+  unsigned int ucode = Ustrtoul(markname + 1, &endptr, 16);
+  if (*endptr != 0) error(66, markname); else return ucode;
+  }
+return 0;
+}
+
+
+
 /*************************************************
 *     Convert character name to Unicode value    *
 *************************************************/
@@ -345,6 +383,7 @@ return TRUE;
 }
 
 
+
 /*************************************************
 *        Find the width of a character           *
 *************************************************/
@@ -378,6 +417,11 @@ font. */
 
 if (c == ZERO_SPACE || c == BREAK_PERMIT || c == NO_BREAK_HERE) return 0;
 
+/* The "hard space" character, U+00A0, does not appear in the AFM files for the
+standard fonts. It is printed as a space, of course. */
+
+if (c == HARD_SPACE && af->stdencoding) c = ' ';
+
 /* The most common, low-numbered characters, have their widths in a table.
 Unknown characters have a characteristic (unreasonable) width. We do not use a
 negative value, because there are fonts that contain characters with negative
@@ -417,7 +461,7 @@ while (top > bot)
 
 if (top <= bot)   /* Character not available */
   {
-  *chtype = CHTYPE_UNKNOWN;
+  if (chtype != NULL) *chtype = CHTYPE_UNKNOWN;
   return 0;
   }
 
@@ -757,6 +801,7 @@ item *i;
 item *stop_at = NULL;
 paramstr *pm;
 vfontstr *vfontstack[FONTSTACKMAX + 1];
+int bullet = -1;
 int section_nest_depth = 0;
 int ss_pinflags = 0;
 int fontstackptr = 0;
@@ -765,6 +810,9 @@ int fontcolour = 0;
 int fontstatestack[FONTSTACKMAX + 1];
 int fontcolourstack[FONTSTACKMAX + 1];
 int fontgroupstack[FONTSTACKMAX + 1];
+BOOL initemizedlist = FALSE;
+BOOL inorderedlist = FALSE;
+BOOL startitem = FALSE;
 
 vfontstr *current_vfont =
   (fonts == FONTS_TOC)?      &toc_maintext_vfont :
@@ -775,6 +823,10 @@ vfontstr *current_vfont =
 
 DEBUG(D_any) debug_printf("Assigning fonts\n");
 
+/* Initialize list parameters that can change during the scan */
+
+pin_default_list_params();
+
 /* Handle an <index> item at the start */
 
 if (Ustrcmp(item_list->name, "index") == 0)
@@ -789,7 +841,8 @@ for (i = item_list; i != NULL; i = i->next)
   {
   fontelstr *fe;
   uschar *name = i->name;
-  if (Ustrcmp(i->name, "#FILENAME") == 0)
+
+  if (Ustrcmp(name, "#FILENAME") == 0)
     {
     read_filename = i->p.string;
     continue;
@@ -802,11 +855,13 @@ for (i = item_list; i != NULL; i = i->next)
   if (i == stop_at || Ustrcmp(name, "index") == 0) break;
 
   /* The only processing instructions that currently matter here are changes to
-  the size of sub/superscript fonts. */
+  the size of sub/superscript fonts, and changes to itemized and ordered list
+  parameters. */
 
-  if (Ustrcmp(i->name, "?sdop") == 0)
+  if (Ustrcmp(name, "?sdop") == 0)
     {
     pin_change_font_assign(i);
+    pin_change_list_params(i);
     continue;
     }
 
@@ -848,21 +903,27 @@ for (i = item_list; i != NULL; i = i->next)
     continue;
     }
 
-  /* Keep track of section nesting and handle relevant font change terminators.
-  Footnotes can occur only in main body text, so there's no need to stack the
-  "fonts" variable - we know what it is going to be. */
+  /* Keep track of itemized and ordered lists and section nesting, and handle
+  relevant font change terminators. Footnotes can occur only in main body text,
+  so there's no need to stack the "fonts" variable - we know what it is going
+  to be. */
 
   if (Ustrcmp(name, "/") == 0)
     {
     item *pt = i->partner;
-    if (ISSECT(pt->name)) section_nest_depth--;
-    if ((pt->flags & IF_FONTSET) != 0)
+    if (Ustrcmp(pt->name, "itemizedlist") == 0) initemizedlist = FALSE;
+    else if (Ustrcmp(pt->name, "orderedlist") == 0) inorderedlist = FALSE;
+    else
       {
-      if (fontstackptr <= 0) (void)error(20);       /* Hard */
-      current_vfont = vfontstack[--fontstackptr];
-      fontcolour = fontcolourstack[fontstackptr];
-      fontstate = fontstatestack[fontstackptr];
-      fonts = fontgroupstack[fontstackptr];
+      if (ISSECT(pt->name)) section_nest_depth--;
+      if ((pt->flags & IF_FONTSET) != 0)
+        {
+        if (fontstackptr <= 0) (void)error(20);       /* Hard */
+        current_vfont = vfontstack[--fontstackptr];
+        fontcolour = fontcolourstack[fontstackptr];
+        fontstate = fontstatestack[fontstackptr];
+        fonts = fontgroupstack[fontstackptr];
+        }
       }
     }
 
@@ -1087,17 +1148,81 @@ for (i = item_list; i != NULL; i = i->next)
     i->flags |= IF_FONTSET;                 /* Remember for reset */
     }
 
-  /* Assign the current font to text items. There's a fudge for footnote key
-  references and definitions. */
+  /* Keep track of itemized and ordered lists. Note the start of each item in
+  such as list so that we can make sure the right font is available. */
+
+  else if (Ustrcmp(name, "itemizedlist") == 0)
+    {
+    initemizedlist = TRUE;
+    bullet = font_get_bullet(i);
+    }
+
+  else if (Ustrcmp(name, "orderedlist") == 0) inorderedlist = TRUE;
+  else if (Ustrcmp(name, "listitem") == 0) startitem = TRUE;
+
+  /* Assign the current font to text items. Cross reference text may already
+  have its colour set. There's a fudge for footnote key references and
+  definitions. */
 
   else if (Ustrcmp(name, "#PCDATA") == 0)
     {
-    i->p.txtblk->colour = fontcolour;
+    if ((i->p.txtblk->pin_flags & (PIN_XREF|PIN_IREF)) == 0 ||
+         i->p.txtblk->colour == -1)
+      i->p.txtblk->colour = fontcolour;
+
     if ((i->p.txtblk->pin_flags & (PIN_FNKEYREF|PIN_FNKEYDEF|PIN_FNREFREF))
          != 0)
       i->p.txtblk->vfont = font_used(&footnote_key_vfont, NULL);
     else
       i->p.txtblk->vfont = font_used(current_vfont, NULL);
+
+    /* Handle tags at the start of items in itemized and ordered lists. At the
+    moment, we stick with the family and size of the main or footnote fonts,
+    but there are parameters to change the type, so we must ensure that a
+    suitable font exists. */
+
+    if (startitem && (initemizedlist || inorderedlist))
+      {
+      int fonttype = initemizedlist? ilist_fs : olist_fs;
+      vfontstr *lf = (fonts == FONTS_FOOTNOTE)?
+        &footnote_maintext_vfont : &maintext_vfont;
+
+      if (lf->type != fonttype)
+        {
+        vfontstr *v = misc_malloc(sizeof(vfontstr));
+        v->next = NULL;
+        v->family = lf->family;
+        v->type = fonttype;
+        v->size = lf->size;
+        v->leading = lf->leading;
+        v->pnumber = -1;
+        v->afont = NULL;
+        memset(v->sfont, 0, sizeof(v->sfont));
+        if ((lf = font_used(v, NULL)) != v) free(v);
+        }
+
+      /* Having now found the font, for an itemized list with an overriding
+      bullet mark, we must ensure that an appropriate auxiliary font is set up
+      if necessary for the bullet character(s). Finding a character's width
+      will get all the necessary work done. If bullet == 0, set up the default
+      list of bullet characters. */
+
+      if (initemizedlist && bullet >= 0)
+        {
+        lf = font_used(lf, NULL);       /* Needed in case bypassed above */
+        font_loadtables(lf->afont);
+        if (bullet > 0)
+          (void)font_charwidth(bullet, lf, NULL);
+        else
+          {
+          int *xp = bullets_default;
+          while (*xp != 0) (void)font_charwidth(*xp++, lf, NULL);
+          }
+        bullet = -1;
+        }
+
+      startitem = FALSE;
+      }
     }
   }
 
