@@ -2,7 +2,7 @@
 *          sdop - Simple DocBook Processor       *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 2009, 2010, 2011 */
+/* Copyright (c) Philip Hazel, 2009, 2010, 2011, 2012 */
 
 /* I wrote this because I was frustrated at the lack of free software for
 processing DocBook which could produce really nice typographic PostScript
@@ -12,8 +12,8 @@ output. Much of the text formatting logic is taken from SGCAL. */
 #ifndef INCLUDED_SDOP_H
 #define INCLUDED_SDOP_H
 
-#define SDOP_VERSION "0.70"
-#define SDOP_DATE    "26-January-2011"
+#define SDOP_VERSION "0.71"
+#define SDOP_DATE    "18-May-2012"
 
 /* General header file for all modules */
 
